library ASTextField;

import 'package:flutter/material.dart';
import 'Constants.dart';
import 'GeneralFunctions.dart';

// ignore: must_be_immutable
class ASTextField extends StatelessWidget {
  //TEXT FIELD HINT
  final String hint;

  //TEXT FIELD ICON PATH
  final String iconPath;

  //TEXT FIELD HINT COLOR
  Color hintColor = AppColors.commonTextHintColor;

  //TEXT FIELD BELOW LINE COLOR
  Color lineColor = AppColors.commonTextLineColor;

  //INIT TEXT FIELD
  final inputTextFieldValue;

  //INPUT TYPE FOR TEXT FIELD - NUMBER, PASSWORD, EMAIL, TEXT
  InputType textFieldInputType;

  //FIELD VALIDATION TYPE, TO CHECK AND VALIDATE FIELD ON BASED ON EMAIL, PASSWORD, MOBILE, ETC.
  TextFieldValidationType validationType;

  //CONSTRUCTOR
  ASTextField(
      this.hint, this.iconPath, this.inputTextFieldValue, this.validationType,
      [this.hintColor, this.lineColor, this.textFieldInputType]);

  //FOR FOCUS TO TEXT FIELD
  final FocusNode focus = new FocusNode();

  //STORE PASSWORD TO COMPARE WITH CONFIRM PASSWORD
  //STATIC IS USED BECAUSE strPassword'S VALUE REMAIN WHILE INITIALIZE AND VALIDATE CONFIRM PASSWORD
  static String strPassword = '';

  @override
  Widget build(BuildContext context) {
    return new Container(
      width: MediaQuery.of(context).size.width,
      margin: const EdgeInsets.only(left: 25.0, right: 25.0),
      alignment: Alignment.center,
      child: Stack(
        children: <Widget>[
          Container(
            child: Align(
              //RIGHT SIDE BUTTON FOR CLICK AND NAVIGATE TO NEXT SCREEN
              alignment: Alignment.topLeft,
              child: Padding(
                padding: const EdgeInsets.only(top: 16),
                child: Image.asset(
                  iconPath,
                  width: 16.0,
                  height: 16.0,
                ),
              ),
            ),
          ),
          Container(
            width: MediaQuery.of(context).size.width,
            child: Align(
              //RIGHT SIDE BUTTON FOR CLICK AND NAVIGATE TO NEXT SCREEN
              alignment: Alignment.topRight,
              child: TextFormField(
                style: setFont(FontWeight.normal, FontStyle.normal, 15),
                focusNode: focus,
                controller: inputTextFieldValue,
                textAlign: TextAlign.start,
                textAlignVertical: TextAlignVertical.center,
                //SET KEYBOARD TYPE FOR - NUMBER, TEXT, EMAIL, ETC.
                keyboardType: getKeyboardType(),
                //TEXT FIELD TYPE PASSWORD or NOT, obscureText = true MEANS PASSWORD TYPE
                obscureText:
                    ((GeneralFunctions.textFieldInputType(textFieldInputType) ==
                            TextInputType.visiblePassword)
                        ? true
                        : false),
                validator: validateTextField,
                decoration: InputDecoration(
                  contentPadding: const EdgeInsets.only(left: 25.0, bottom: 16),
                  hintText: hint,
                  hintStyle: TextStyle(color: hintColor, fontSize: 15),
                  errorStyle: TextStyle(fontSize: 15),
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  //FUNCTION RETURNS TEXT FILED TO GET ALL PARAMETERS OF TEXT FILED
  TextEditingController getFieldValue() {
    return inputTextFieldValue;
  }

  getKeyboardType() {
    return GeneralFunctions.textFieldInputType(textFieldInputType);
  }

  static setFont(FontWeight weight,
      [FontStyle style = FontStyle.normal, double fontSize = 15]) {
    return TextStyle(
        fontFamily: AppFont.roboto,

        //FontWeight - normal OR bold
        fontWeight: FontWeight.normal,

        //FontStyle - normal OR italic
        fontStyle: style,

        //Text font size
        fontSize: fontSize);
  }

  //CHECK - VALIDATION ON INDIVIDUAL FIELD TYPE
  String validateTextField(String value) {
    //VALIDATION FOR USER-NAME
    if (validationType == TextFieldValidationType.userName) {
      if (value.length == 0) {
        return 'Please enter username';
      }
    }

    //VALIDATION FOR EMAIL
    if (validationType == TextFieldValidationType.email) {
      return validateEmail(value);
    }

    //VALIDATION FOR MOBILE-NUMBER
    if (validationType == TextFieldValidationType.mobileNumber) {
      if (value.length < 10) {
        return 'Please enter valid mobile number';
      }
    }

    //VALIDATION FOR COMPANY-NAME
    if (validationType == TextFieldValidationType.companyName) {
      if (value.length == 0) {
        return 'Please enter company name';
      }
    }

    //VALIDATION FOR SUB-DOMAIN
    if (validationType == TextFieldValidationType.subDomain) {
      if (value.length < 3) {
        return 'SubDomain must be upto 3 characters';
      }
    }

    //VALIDATION FOR PASSWORD
    if (validationType == TextFieldValidationType.password) {
      if (value.length < 6) {
        strPassword = '';
        return 'Password should contains atleast 6 characters';
      } else {
        //STORE PASSWORD TO COMPARE WITH CONFIRM PASSWORD
        strPassword = value;
      }
    }

    //VALIDATION FOR CONFIRM PASSWORD
    if (validationType == TextFieldValidationType.confirmPassword) {
      if (strPassword.isEmpty) {
        if (value.length < 6) {
          return 'Password should contains atleast 6 characters';
        }
      } else {
        if (strPassword != value) {
          return 'Password and confirm password is not matched';
        }
      }
    }

    //MAKE SURE TO RETURN NULL, IF ALL VALIDATION IS PROPER
    return null;
  }
}

String validateEmail(String value) {
  if (value.isEmpty) {
    // The form is empty
    return "Enter email address";
  }
  // This is just a regular expression for email addresses
  String p = "[a-zA-Z0-9\+\.\_\%\-\+]{1,256}" +
      "\\@" +
      "[a-zA-Z0-9][a-zA-Z0-9\\-]{0,64}" +
      "(" +
      "\\." +
      "[a-zA-Z0-9][a-zA-Z0-9\\-]{0,25}" +
      ")+";
  RegExp regExp = new RegExp(p);

  if (regExp.hasMatch(value)) {
    // So, the email is valid
    return null;
  }

  // The pattern of the email didn't match the regex above.
  return 'Email is not valid';
}
